;***************************************************************************
; 
; File Name		:'EEPROM.asm"
; Title			:
; Date			:
; Version		:
; Support telephone	:765 287 1987  David B. VanHorn
; Support fax		:765 287 1989
; Support Email		:dvanhorn@cedar.net
; Support Snail		;1104 E 13th St, Muncie IN 47302
; Target MCU		:AT90S8515
; 
;***************************************************************************;
;	D E S C R I P T I O N
;
;Example code for using the onboard EEPROM including safe read and write
;
;***************************************************************************;
;	M O D I F I C A T I O N   H I S T O R Y 
;
;
;       rev.      date    who   why
;	----	--------  ---	------------------------------------------
;	0.01	98.11.30  dvh	Creation
;	0.02	99.10.21  dvh	Just goes to show you how long a simple
;				bug can go undetected. I was setting the
;				wrong bits in previous versions!
;				(Don't you guys TEST this stuff?? :)
;				Updated to use the bit names, and added 
;				readback verification on write
;	0.03	00.10.10 dvh	Found a bug in the ICE200 WRT the EEPROM flags
;				Made things a little clearer.
;
;********************************************************************
;
.dseg
EEPROM_Timer:	.byte	1	;Used for timeout on waiting for writes to complete
.cseg
.equ	Max_EE_Wait =	5	;mS before we give up waiting for a write to complete
				;You could set this smaller, but watch out.
.equ	EE_Pattern = $AAh	;
.equ	Disp_Speed = 5		;Ms delay before continue
			
;          00000000011111111112222222222333
;          12345678901234567890123456789012
EE_Blown:
.db       "EEPROM BLOWN!!! XXX SB XX IS XX ",0
EE_Paving:
.db       "PAVING EEPROM   Address=XXX     ",0
EE_Testing:
.db       "Testing EEPROM  Address=XXX     ",0
EE_Ok:
.db       "EEPROM Test OK!  Version FOUR.0 ",0

;********************************************************************
;Init EEPROM to all AAh
;********************************************************************

EE_Pave:
	ldi	XL,$13			;EEPROM address
	ldi	XH,$01			;
	rcall	Read_EE			;See if it's already paved
	cpi	TEMP,EE_Pattern		;
	breq	EE_Pave_Done		;

	ldi	ZL,low(EE_Paving*2)	;Make the Z reg point at the table
	ldi	ZH,high(EE_Paving*2)	;preparing for the LPM instruction
	rcall	VFD_STR_OUT		;String to buf, buf to VFD
	rcall	Check_VFD_Display 	;

	ldi	XL,$00			;EEPROM address
	ldi	XH,$00			;
	ldi	TEMP,EE_Pattern		;Data

EE_Pave_Loop:
	rcall	EE_Disp_Test_1		;Write address into display memory
	ldi	TEMP,EE_Pattern		;Data
	rcall	Save_EE			;Write it to EEPROM
					;Incs X, so it's ready to go for
					;sequential operations
					;Sets Carry flag if problems

	brcc	EE_Pave_Again		;If succesful, the do the next.
	nop				;Otherwise at least a breakpoint

	ldi	ZL,low(EE_Blown*2)	;Make the Z reg point at the table
	ldi	ZH,high(EE_Blown*2)	;preparing for the LPM instruction
	rcall	VFD_STR_OUT		;String to buf, buf to VFD
	rcall	Check_VFD_Display 	;
	
EE_Pave_Here:
	rcall	Timed_Smack		;
	rjmp	EE_Pave_Here		;


EE_Pave_Again:
	cpi	XH,$02			;If XH=02, the XL has to be 00
	brne	EE_Pave_Loop		;
EE_Pave_Done:
	ret

;*************************************************************************
;
;We're going to put some values into these two messages, defined above, and 
;then update the display.
;
;           00000000001111111111222222222233
;           01234567890123456789012345678901
;.db       "PAVING EEPROM   Address=XXX     ",0

EE_Disp_Test_1:

	;Since we're bypassing the buffer management, and editing 
	;chars directly, we need to point past the length and
	;size bytes.
	
	push	XL			;
	push	XH			;
	
	ldi	ZL,low(VFD_Out_Buf+2)	;Make the Z reg point at the table
	ldi	ZH,high(VFD_Out_Buf+2)	;preparing for the LPM instruction
	;add offset to point at the XXX section
	clc				;
	ldi	TEMP2,24		;Offset, to point at XXX
	add	ZL,TEMP2		;
	add	ZH,ZERO			;

	mov	TEMP2,XH		;Get the high byte
	rcall	EE_Hex2ASC		;
	st	Z+,TEMP2		;Save it
	
	mov	TEMP2,XL		;Get the low byte
	swap 	TEMP2			;I want the high nybble
	rcall	EE_Hex2ASC		;
	st	Z+,TEMP2		;Save it

	mov	TEMP2,XL		;Get the low byte again
	rcall	EE_Hex2ASC		;
	st	Z+,TEMP2		;Save it

	;Mark display dirty
	lds	TEMP_BITFLAGS,VFD_Bitflags;
	ori	TEMP_Bitflags,$40	;Flag the display dirty
	sts	VFD_Bitflags,TEMP_Bitflags

	rcall	Check_VFD_Display 	;
	ldi	TEMP,Disp_Speed		;
	rcall	MS_Delay		;
	pop	XH
	pop	XL
	ret


;           00000000001111111111222222222233
;           01234567890123456789012345678901
;.db       "EEPROM BLOWN!!! XXX SB XX IS XX ",0

EE_Disp_Test_2:

	push	XL
	push	XH

	;Since we're bypassing the buffer management, and editing 
	;chars directly, we need to point past the length and
	;size bytes.
	ldi	ZL,low(VFD_Out_Buf+2)	;Make the Z reg point at the table
	ldi	ZH,high(VFD_Out_Buf+2)	;preparing for the LPM instruction
	;add offset to point at the XXX section
	clc				;
	ldi	TEMP2,16		;Offset, to point at XXX
	add	ZL,TEMP2		;
	adc	ZH,ZERO			;Now pointing at the first "X"

	;Dec X, because it's pointing to the NEXT byte
	ld	TEMP2,-X		;Trash the data, but move 
					;the pointer!

	mov	TEMP2,XH		;Get the high byte
	rcall	EE_Hex2ASC		;
	st	Z+,TEMP2		;Save it, then inc pointer

	mov	TEMP2,XL		;Get the low byte
	swap 	TEMP2			;I want the high nybble
	rcall	EE_Hex2ASC		;
	st	Z+,TEMP2		;Save it, then inc pointer

	mov	TEMP2,XL		;Get the low byte again
	rcall	EE_Hex2ASC		;
	st	Z+,TEMP2		;Save it, then inc pointer (19)				;

	ldi	TEMP2,4			;Offset, to point SB XX (23)
	clc				;
	add	ZL,TEMP2		;
	adc	ZH,ZERO			;

	ldi	TEMP2,EE_Pattern	;
	swap 	TEMP2			;I want the high nybble
	rcall	EE_Hex2ASC		;
	st	Z+,TEMP2		;Save it (24)

	ldi	TEMP2,EE_Pattern	;
	andi	TEMP2,$0F		;Mask it
	add	TEMP2,TEMP3		;Convert to ASCII
	st	Z+,TEMP2		;Save it (25)

	clc				;
	ldi	TEMP2,4			;Offset, to point IS XX
	add	ZL,TEMP2		;
	adc	ZH,ZERO			;

	rcall	Read_EE			;Output in TEMP, incs the pointer
	mov	TEMP2,TEMP		;Hex2asc operates on temp2
	swap 	TEMP2			;I want the high nybble
	rcall	EE_Hex2ASC		;
	st	Z+,TEMP2		;Save it

	ld	TEMP,-X			;Put that pointer back!
	rcall	Read_EE			;
	mov	TEMP2,TEMP		;
	rcall	EE_Hex2ASC		;
	st	Z+,TEMP2			;Save it

	;Mark display dirty
	lds	TEMP_BITFLAGS,VFD_Bitflags;
	ori	TEMP_Bitflags,$40	;Flag the display dirty
	sts	VFD_Bitflags,TEMP_Bitflags

	rcall	Check_VFD_Display 	;
	
	pop	XH			;
	pop	XL			;
	ldi	TEMP,Disp_Speed		;
	rcall	MS_Delay		;
	ret

;
;Converts the low nybble of a byte to 0-9 or A-F
;
EE_Hex2ASC:
	andi	TEMP2,$0F	;Mask off junk
	cpi	TEMP2,$0A	;
	brge	EE_Hex_Alpha	;
	
EE_Hex_Num:
	ldi	TEMP3,$30	;
	add	TEMP2,TEMP3	;
	ret
	
EE_Hex_Alpha:
	ldi	TEMP3,$37	;
	add	TEMP2,TEMP3	;
	ret			;



;********************************************************************
;Here, we take each setting from SRAM or Registers, and put them into
;EEPROM for safekeeping
;********************************************************************

Check_EE:

	ldi	ZL,low(EE_Testing*2)	;Make the Z reg point at the table
	ldi	ZH,high(EE_Testing*2)	;preparing for the LPM instruction
	rcall	VFD_STR_OUT		;String to buf, buf to VFD
	rcall	Check_VFD_Display 	;

	ldi	XL,$00			;
	ldi	XH,$00			;

Check_EE_Again:
	rcall	EE_Disp_Test_1		;Show where we are working
	rcall	Read_EE			;Write it to EEPROM
					;Incs X, so it's ready
	cpi	TEMP,EE_Pattern		;
	breq	Check_EE_ok		;
	
	;If bad, then say so

	ldi	ZL,low(EE_Blown*2)	;Make the Z reg point at the table
	ldi	ZH,high(EE_Blown*2)	;preparing for the LPM instruction
	rcall	VFD_STR_OUT		;String to buf, buf to VFD
	rcall	EE_Disp_Test_2		;Show where the fault is
	rcall	Check_VFD_Display 	;
	ld	TEMP,-X			;Put the pointer back
	ldi	TEMP,EE_Pattern		;Try to restore the cell for
	rcall	Save_EE			;next time (dosen't display)

	;Hang here till someone notices
Check_EE_Hung:
	rcall	Timed_Smack		;
	rjmp	Check_EE_Hung		;	

	;Otherwise, inc and check another
Check_EE_ok:
	cpi	XH,$02			;If XH=02, the XL has to be 00
	brne	Check_EE_Again		;

	;If we make it, then say so
EE_Test_Done:
	ldi	ZL,low(EE_OK*2)		;Make the Z reg point at the table
	ldi	ZH,high(EE_OK*2)	;preparing for the LPM instruction
	rcall	VFD_STR_OUT		;String to buf, buf to VFD
	rcall	Check_VFD_Display 	;
	ldi	TEMP,250		;
	rcall	MS_Delay		;
	ldi	TEMP,250		;
	rcall	MS_Delay		;
	ldi	TEMP,250		;
	rcall	MS_Delay		;
	ldi	TEMP,250		;
	rcall	MS_Delay		;
	cbi	DDRB,0			;Set bit 0 as output
	cbi	PORTB,0			;Take it low, shutting off my
					;own power.
	ret				;Not likely...
	

;********************************************************************
;Put a single byte in EEPROM, safely per manual, with readback verify
;Call with pointer to EEPROM in X, data in TEMP.
;Increments X, Trashes TEMP2
;********************************************************************
;
Save_EE:

	;APPARENT EMULATOR BUG IN THE ICE200 REQUIRES THAT
	;YOU POLL FOR /EEWE AND /EERE BEFORE WRITING, AFTER A READ

	rcall	Timed_Smack		;In case the loop takes a while
	in	TEMP2,EECR		;Poll for ready
	andi	TEMP2,3			;All three bits must be zero
	brne	Save_EE			;Loop till zero

	out	EEARH,XH		;Point at the current cell
	out	EEARL,XL		;
	out	EEDR,TEMP		;Data's ready to be saved

	;WARNING: This section cannot be single-stepped in the
	;emulator due to timing constraints.
	cli				;No ints now
	sbi	EECR,2 ;EEMWE		;Set master write enable=1
	sbi	EECR,1 ;EEWE		;Set write enable to 1, starts the write
	sei				;Ok, now you can bother me.
	;End of critical section

	adiw	XL,1			;Increment the address for nest call

	;rjmp	Save_EE_Ok		;If you don't want readback, then
					;all you have to do is uncomment this line


	;Now, we have to wait for the write to complete
Save_EE_Verify:
	ldi	TEMP2,Max_EE_Wait	;Max mS to wait
	sts	EEPROM_Timer,TEMP2	;
Save_EE_V_Loop:
	rcall	Timed_Smack		;In case we wait through a WDT
	lds	TEMP2,EEPROM_Timer	;
	and	TEMP2,TEMP2		;Is it zero yet?
	breq	Save_EE_Bad2		;If so, then signal failure >20ms to write

	in	TEMP2,EECR		;Poll for EECR,EEWE=1 before continuing!
	andi	TEMP2,1 ;EEWE		;
	brne	Save_EE_V_Loop		;Potential to loop forever.


	;Time to read the data
	;Address is already set, so just read it back
	sbi	EECR,1 ;EERE		;Set EE Read Enable
	in	TEMP2,EEDR		;Get the data

	;If unequal, then signal failure
	cp	TEMP,TEMP2		;
	brne	Save_EE_Bad		;

	;otherwise
	clc				;Indicate success
	rjmp	Save_EE_Exit		;

	;Redundant code, but it lets me breakpoint and know which way it failed.

Save_EE_Bad:
	sec				;Data didn't match
	
Save_EE_Bad2:
	sec				;timeout on readback

Save_EE_Exit:
	ldi	TEMP2,$FF		;Point at non-existent location
	out	EEARL,TEMP2		;when not in active use.
	out	EEARH,TEMP2		;
	ret				;
;
;********************************************************************
;Gets a single byte from EEPROM, safely
;********************************************************************
;
Read_EE:
	rcall	Timed_Smack		;
	in	TEMP2,EECR		;Poll for EECR,2=0 before continuing!
	andi	TEMP2,$03		;Not sure you need ALL flags=0 but it's harmless
					;See bug description in Write_EE
	brne	Read_EE			;Potential to loop forever.

	out	EEARH,XH		;Set up the address
	out	EEARL,XL		;
	sbi	EECR,EERE		;Set EE Read Enable
	in	TEMP,EEDR		;Get the data

	;If you're REALLY paranoid, you could do a second read, and compare
	;here, as above, but if the cell stored junk, then you'll likely
	;just get the same junk twice.

	adiw	XL,1			;Inc the pointer

	ldi	TEMP2,$FF		;Point at non-existent location
	out	EEARL,TEMP2		;when not in active use.
	out	EEARH,TEMP2		;

	ret				;


