;***************************************************************************
; 
; File Name		:'MAIN.asm"
; Title			:
; Date			:
; Version		:coded for the atmel WAVRASM assembler
; Support telephone	:765 287 1987  David B. VanHorn
; Support fax		:765 287 1989
; Support Email		:dvanhorn@cedar.net
; Support Snail		;1104 E 13th St, Muncie IN 47302
; Target MCU		:AT90S8515
; 
;***************************************************************************;
;	D E S C R I P T I O N
;
; An EEPROM test routine.
; After reset, we pull PB0 high.
; This inits the EEPROM if not already done (could be improved).
; During the "pave" operation, the cells are read back, and if not programmable,
; the routine hangs, displaying the bad cell location.  If it inits ok, then we test
; the EEPROM cells that they read back the right value. 
; Any bad cell is displayed, until powerdown, and the SW over-writes it with the correct
; pattern for the next run-through.
; 
; If we make it through the entire array, then I pull PB0 low
; This can be used to cycle power to the chip, with an external, AC coupled pass transistor.
; If power dosen't dissapear, then I just loop back and read-test again.
;
; Output is to the Futaba VFD, cause I like how they look :)
; It's easily hackable to the LCD routines, just use the ones from "Getting Started", and 
; fix up the buffer inits, and other references to VFD.
;
;***************************************************************************;
;	M O D I F I C A T I O N   H I S T O R Y 
;
;
;       rev.      date    who   why
;	----	--------  ---	------------------------------------------
;	0.01	00.10.12  dvh	Creation 
;
;********************************************************************
;This application consists of seventeen files. They should all be in the
;same directory, or you will have to edit the .include directives 
;below to make them point to the right locations.
;
;Main.asm (You're looking at it)  The main structure of the program
;8515def.inc	Port and pin definitions, supplied by ATMEL
;Equates.inc  	Where constants get defined as words, to make life easier
;Isr.asm	Interrupt Service Routines
;Init.asm	Machine initialization
;Memory.asm	Buffer management (MUCH REVISED!)
;Random.asm	A 19 bit PseudoNoise generator
;Watchdog.asm	Watchdog timer code
;VFD.asm	Handler for Futaba Vaccum Flourescent Displays
;EEPROM.asm	Handler for the internal EEPROM, coded to save and recall
;
;********************************************************************
;Physical resources.
;
;This application is designed for an AVR demo board, with 8.00 MHz crystal.
;For bonus points, modify it to use a 4 MHz xtal, without changing the speed 
;that anything runs at. That's a good first project!
;
;********************************************************************
;
;Logical equates to make life easier
;
.include "EQUATES.INC" ;The basics, naming of registers etc.
;It's far easter to remember what "LOOP" is for, than "R15"
;
.cseg ;This tells the assembler that what follows is code, and goes in ROMspace
;
;********************************************************************
;
;The actual beginning of the code.
;INIT the machine, start the ISRs, and exit into idle
;On reset, the 8515 jumps to 0000, which contains a jump vector (see ISR.ASM)
;
.include "Isr.asm"	;Restart vector and Interrupt service routines are here
.include "Init.asm"	;Initialization of timers,ports,and ext hdw.
;
;***************************************************************************************
;
Idle:	

	rcall	Random			;Result in RAND1,RAND2,and RAND3
	;
	
	rcall	Check_EE		;
	
	
	;********************************************************
	;
	;If you use Ping, make sure it's assigned to an unused output pin, otherwise
	;it will cause problems with whatever else is trying to use that port.
	;
	;ldi	TEMP,$01		;A ping every time we loop.
	;rcall	Ping			;

	rcall	Timed_Smack		;Feed the watchdog, if it's needed (Watchdog.asm)

	rjmp	Idle			;

;*************************************************************************************
;I really hate doing things like this, but so far, I haven't hit on
;a good alternative.

MS_Delay:
	push 	TEMP2
	push	TEMP3

MS_Delay_1:
	ldi	TEMP2,$FF
MS_Delay_2:
	ldi	TEMP3,$10
MS_Delay_3:
	dec	TEMP3
	brne	MS_Delay_3

	dec	TEMP2
	brne	MS_Delay_2

	wdr			;Because otherwise we might get whacked
	dec	TEMP
	brne	MS_Delay_1

	pop	TEMP3
	pop	TEMP2
	ret	
;
;********************************************************************
;
;A terribly useful little diagnostic hack, provided you've got a scope.
;Just call Ping with a number in TEMP, and it will give you N pulses.
;This is very useful for figuring which branch of a program is taken 
;on live hardware, when you haven't got an emulator, and simulation 
;isn't really an option.
;
.equ	Ping_Data=PORTA		;Which port will we use?
.equ	Ping_DDR=PORTA		;Must match above
.equ	Ping_Bit=1		;Ping output bit
;
Ping:
	sbi	Ping_DDR,Ping_Bit	;Make sure it's an output
	sbi	Ping_Data,Ping_Bit	;I hope your scope can resolve to 125nS
	cbi 	Ping_Data,Ping_Bit	;
	dec	TEMP			;One less ping
	brne	Ping			;Are we done? If not, then ping again
	ret				;If so, bye!
;********************************************************************
;External Routines
;
.include "EEPROM.asm"	;EEPROM save and recall
.include "Memory.asm"	;Memory allocation, buffer handlers
.include "Random.asm"	;Pseudorandom generator, Specifically, a 19
			;bit maximal length generator.
.include "VFD.asm"	;VFD display controller
.include "Watchdog.asm"	;Watchdog timer handler
;
;***************************************************************************
;
.db	"COPYRIGHT 1997-2000 David VanHorn",CR,LF
.db	"ALL RIGHTS RESERVED",CR,LF
;
;***************************************************************************
